@@ -2,6 +2,7 @@
  *  
  *   This file is part of the libcrackle library.
  *       Copyright (c) 2008-2014 Lost Island Labs
+ *           <info@utopiadocs.com>
  *   
  *   The libcrackle library is free software: you can redistribute it and/or
  *   modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
@@ -28,6 +29,7 @@ extern "C" {
 #endif
 
     SpineDocument new_CrackleDocument(const char *filename, SpineError *error);
+    SpineDocument new_CrackleDocumentFromBuffer(const char *buffer, size_t size, SpineError *error);
 
 #ifdef __cplusplus
 }
