@@ -2,6 +2,7 @@
 #   
 #    This file is part of the Utopia Documents application.
 #        Copyright (c) 2008-2014 Lost Island Labs
+#            <info@utopiadocs.com>
 #    
 #    Utopia Documents is free software: you can redistribute it and/or modify
 #    it under the terms of the GNU GENERAL PUBLIC LICENSE VERSION 3 as
@@ -40,6 +41,7 @@ import kend.model
 import re
 import socket
 import spineapi
+import traceback
 import urllib2
 import utopia.document
 from lxml import etree
@@ -90,10 +92,43 @@ class MetadataStorage(utopia.document.Annotator):
         kend.client.Client().submitMetadata(uri, doc).metadata
 
 
+
+
 class Metadata(utopia.document.Annotator):
     """Assemble standard metadata"""
 
-    def prepare(self, document):
+    def on_load_event(self, document):
+
+        # Keep track of errors so that we can inform the user
+        def add_error(component, method, category = None, message = None, exception = None):
+            if exception is not None:
+                if isinstance(exception, urllib2.URLError) and isinstance(exception.reason, socket.timeout):
+                    exception = exception.reason
+
+                if isinstance(exception, socket.timeout):
+                    category = 'timeout'
+                    message = 'The server did not respond'
+                elif isinstance(exception, urllib2.HTTPError):
+                    category = 'server'
+                    message = unicode(getattr(exception, 'reason', 'The server did not respond as expected'))
+                elif isinstance(exception, urllib2.URLError):
+                    category = 'connection'
+                    message = unicode(getattr(exception, 'reason', 'The server could not be found'))
+            error = spineapi.Annotation()
+            error['concept'] = 'Error'
+            error['property:component'] = component
+            error['property:method'] = method
+            error['property:category'] = category
+            if message is not None:
+                error['property:message'] = message
+            document.addAnnotation(error, 'errors.metadata')
+        def add_success(component, method):
+            error = spineapi.Annotation()
+            error['concept'] = 'Success'
+            error['property:component'] = component
+            error['property:method'] = method
+            error['property:category'] = 'success'
+            document.addAnnotation(error, 'errors.metadata')
 
         metadata = {
             'scraped': {},
@@ -127,18 +162,24 @@ class Metadata(utopia.document.Annotator):
         arxivid = common.arxiv.scrape(document)
         if arxivid is not None:
             metadata['scraped']['arxivid'] = arxivid
-            arxiv_results = common.arxiv.resolve(arxivid)
-            if arxiv_results is not None:
-                arxiv_results.update({':whence': 'arxiv', ':weight': 10})
-                common.utils.store_metadata(document, **arxiv_results)
+            try:
+                arxiv_results = common.arxiv.resolve(arxivid)
+                if arxiv_results is not None:
+                    arxiv_results.update({':whence': 'arxiv', ':weight': 10})
+                    common.utils.store_metadata(document, **arxiv_results)
+            except Exception as e:
+                add_error('ArXiv', 'resolve', exception=e)
+                traceback.print_exc()
+            else:
+                add_success('ArXiv', 'resolve')
 
         #################################################################################
         # Fold in the CrossRef data
 
         issn = common.utils.metadata(document, 'issn')
         if title is not None or doi is not None:
-            try:
-                if doi is None:
+            if doi is None:
+                try:
                     xref_results = common.crossref.search(title)
                     if len(xref_results) == 1:
                         xref_title = xref_results[0].get('title')
@@ -152,8 +193,14 @@ class Metadata(utopia.document.Annotator):
                             if len(matches) > 0:
                                 doi = xref_results[0].get('doi')
                                 print 'crossref: accepting resolved doi'
-                if doi is not None:
-                    # What is this DOI's article's title according to crossref?
+                except Exception as e:
+                    add_error('CrossRef', 'search', exception=e)
+                    traceback.print_exc()
+                else:
+                    add_success('CrossRef', 'search')
+            if doi is not None:
+                # What is this DOI's article's title according to crossref?
+                try:
                     xref_results = common.crossref.resolve(doi)
                     xref_results.update({':whence': 'crossref', ':weight': 20})
                     xref_title = xref_results.get('title', '')
@@ -172,9 +219,11 @@ class Metadata(utopia.document.Annotator):
                             else:
                                 print 'crossref: ignoring resolved metadata'
                                 # FIXME should we discard the DOI at this point?
-            except socket.timeout:
-                # Deal with the server being a bit crap FIXME
-                pass
+                except Exception as e:
+                    add_error('CrossRef', 'resolve', exception=e)
+                    traceback.print_exc()
+                else:
+                    add_success('CrossRef', 'resolve')
 
         ###########################################################################################
         # Fold in the PubMed data
@@ -182,81 +231,117 @@ class Metadata(utopia.document.Annotator):
         pmid = common.utils.metadata(document, 'pmid')
         pmcid = common.utils.metadata(document, 'pmcid')
         if pmid is None and doi is not None: # resolve on DOI
-            pmid = common.pubmed.resolve(doi, 'doi')
+            try:
+                pmid = common.pubmed.resolve(doi, 'doi')
+            except Exception as e:
+                add_error('PubMed', 'resolve', exception=e)
+                traceback.print_exc()
+            else:
+                add_success('PubMed', 'resolve')
         if pmid is None and title is not None: # resolve on title
-            pubmed_results = common.pubmed.search(title)
-            pubmed_title = pubmed_results.get('title', '').strip(' .')
-            if len(pubmed_title) > 0:
-                print 'pubmed: resolved title:', pubmed_title.encode('utf8')
-                pubmed_pmid = pubmed_results.get('pmid')
-                print 'pubmed: resolved pmid:', pubmed_pmid
-                if re.sub(r'[^\w]+', ' ', title).strip() == re.sub(r'[^\w]+', ' ', pubmed_title).strip(): # Fuzzy match
-                    print 'pubmed: titles match precisely'
-                    title = pubmed_title
-                    pmid = pubmed_pmid
-                else:
-                    # Accept the pubmed title over the scraped title, if present in the document
-                    matches = document.findInContext('', pubmed_title, '') # Fuzzy match
-                    if len(matches) > 0:
-                        title = matches[0].text()
+            try:
+                pubmed_results = common.pubmed.search(title)
+                pubmed_title = pubmed_results.get('title', '').strip(' .')
+                if len(pubmed_title) > 0:
+                    print 'pubmed: resolved title:', pubmed_title.encode('utf8')
+                    pubmed_pmid = pubmed_results.get('pmid')
+                    print 'pubmed: resolved pmid:', pubmed_pmid
+                    if re.sub(r'[^\w]+', ' ', title).strip() == re.sub(r'[^\w]+', ' ', pubmed_title).strip(): # Fuzzy match
+                        print 'pubmed: titles match precisely'
+                        title = pubmed_title
                         pmid = pubmed_pmid
-                        print 'pubmed: overriding scraped title with pubmed title'
                     else:
-                        print 'pubmed: ignoring resolved title'
+                        # Accept the pubmed title over the scraped title, if present in the document
+                        matches = document.findInContext('', pubmed_title, '') # Fuzzy match
+                        if len(matches) > 0:
+                            title = matches[0].text()
+                            pmid = pubmed_pmid
+                            print 'pubmed: overriding scraped title with pubmed title'
+                        else:
+                            print 'pubmed: ignoring resolved title'
+            except Exception as e:
+                add_error('PubMed', 'search', exception=e)
+                traceback.print_exc()
+            else:
+                add_success('PubMed', 'search')
         if pmid is not None:
-            nlm = common.pubmed.fetch(pmid)
-            if nlm is not None:
-                xml = etree.fromstring(nlm)
+            try:
+                nlm = common.pubmed.fetch(pmid)
+                if nlm is not None:
+                    xml = etree.fromstring(nlm)
 
-                pubmed_authors = []
-                for author in xml.findall('PubmedArticle/MedlineCitation/Article/AuthorList/Author'):
-                    name = u''
-                    lastName = author.findtext('LastName')
-                    forename = author.findtext('ForeName')
-                    if lastName is not None:
-                        name = lastName + u', '
-                    if forename is not None:
-                        name += forename
-                    if len(name) > 0:
-                        pubmed_authors.append(name)
-                if len(pubmed_authors) == 0:
-                    pubmed_authors = None
+                    pubmed_authors = []
+                    for author in xml.findall('PubmedArticle/MedlineCitation/Article/AuthorList/Author'):
+                        name = u''
+                        lastName = author.findtext('LastName')
+                        forename = author.findtext('ForeName')
+                        if lastName is not None:
+                            name = lastName + u', '
+                        if forename is not None:
+                            name += forename
+                        if len(name) > 0:
+                            pubmed_authors.append(name)
+                    if len(pubmed_authors) == 0:
+                        pubmed_authors = None
 
-                pubmed_pmid = xml.findtext('PubmedArticle/MedlineCitation/PMID')
+                    pubmed_pmid = xml.findtext('PubmedArticle/MedlineCitation/PMID')
 
-                common.utils.store_metadata(document, **{
-                    ':whence': 'pubmed',
-                    ':weight': 10,
-                    'raw_pubmed_nlm': nlm,
-                    'authors': pubmed_authors,
-                    'pmid': pubmed_pmid,
-                    'title': xml.findtext('PubmedArticle/MedlineCitation/Article[1]/ArticleTitle'),
-                    'issn': xml.findtext('PubmedArticle/MedlineCitation/Article/Journal/ISSN[1]'),
-                    'doi': xml.findtext('PubmedArticle/PubmedData/ArticleIdList/ArticleId[@IdType="doi"]'),
-                    'pmcid': xml.findtext('PubmedArticle/PubmedData/ArticleIdList/ArticleId[@IdType="pmc"]'),
-                    'pii': xml.findtext('PubmedArticle/PubmedData/ArticleIdList/ArticleId[@IdType="pii"]'),
-                    'publication-title': xml.findtext('PubmedArticle/MedlineCitation/Article/Journal/Title'),
-                    'volume': xml.findtext('PubmedArticle/MedlineCitation/Article/Journal/JournalIssue/Volume'),
-                    'issue': xml.findtext('PubmedArticle/MedlineCitation/Article/Journal/JournalIssue/Issue'),
-                    'year': xml.findtext('PubmedArticle/MedlineCitation/Article/Journal/JournalIssue/PubDate/Year'),
-                    'pages': xml.findtext('PubmedArticle/MedlineCitation/Article[1]/Pagination/MedlinePgn'),
-                    'abstract': xml.findtext('PubmedArticle/MedlineCitation/Article[1]/Abstract/AbstractText'),
-                })
-                pmid = pubmed_pmid or pmid
+                    common.utils.store_metadata(document, **{
+                        ':whence': 'pubmed',
+                        ':weight': 10,
+                        'raw_pubmed_nlm': nlm,
+                        'authors': pubmed_authors,
+                        'pmid': pubmed_pmid,
+                        'title': xml.findtext('PubmedArticle/MedlineCitation/Article[1]/ArticleTitle'),
+                        'issn': xml.findtext('PubmedArticle/MedlineCitation/Article/Journal/ISSN[1]'),
+                        'doi': xml.findtext('PubmedArticle/PubmedData/ArticleIdList/ArticleId[@IdType="doi"]'),
+                        'pmcid': xml.findtext('PubmedArticle/PubmedData/ArticleIdList/ArticleId[@IdType="pmc"]'),
+                        'pii': xml.findtext('PubmedArticle/PubmedData/ArticleIdList/ArticleId[@IdType="pii"]'),
+                        'publication-title': xml.findtext('PubmedArticle/MedlineCitation/Article/Journal/Title'),
+                        'volume': xml.findtext('PubmedArticle/MedlineCitation/Article/Journal/JournalIssue/Volume'),
+                        'issue': xml.findtext('PubmedArticle/MedlineCitation/Article/Journal/JournalIssue/Issue'),
+                        'year': xml.findtext('PubmedArticle/MedlineCitation/Article/Journal/JournalIssue/PubDate/Year'),
+                        'pages': xml.findtext('PubmedArticle/MedlineCitation/Article[1]/Pagination/MedlinePgn'),
+                        'abstract': xml.findtext('PubmedArticle/MedlineCitation/Article[1]/Abstract/AbstractText'),
+                    })
+                    pmid = pubmed_pmid or pmid
 
-                # FIXME I'm sure the above should be in common.pubmed
+                    # FIXME I'm sure the above should be in common.pubmed
+            except Exception as e:
+                add_error('PubMed', 'fetch', exception=e)
+                traceback.print_exc()
+            else:
+                add_success('PubMed', 'fetch')
 
         ###########################################################################################
         # Fold in the PubMedCentral data
         if pmcid is None and doi is not None: # resolve on DOI
-            pmcid = common.pmc.resolve(doi, 'doi')
+            try:
+                pmcid = common.pmc.resolve(doi, 'doi')
+            except Exception as e:
+                add_error('PubMed Central', 'resolve', exception=e)
+                traceback.print_exc()
+            else:
+                add_success('PubMed Central', 'resolve')
         if pmcid is None and pmid is not None: # resolve on PubMed ID
-            pmcid = common.pmc.resolve(pmid, 'pmid')
+            try:
+                pmcid = common.pmc.resolve(pmid, 'pmid')
+            except Exception as e:
+                add_error('PubMed Central', 'resolve', exception=e)
+                traceback.print_exc()
+            else:
+                add_success('PubMed Central', 'resolve')
         if pmcid is not None:
             common.utils.store_metadata(document, **{':whence': 'pmc', ':weight': 10, 'pmcid': pmcid})
-            nlm = common.pmc.fetch(pmcid)
-            if nlm is not None:
-                common.utils.store_metadata(document, **{':whence': 'pmc', ':weight': 10, 'raw_pmc_nlm': nlm})
+            try:
+                nlm = common.pmc.fetch(pmcid)
+                if nlm is not None:
+                    common.utils.store_metadata(document, **{':whence': 'pmc', ':weight': 10, 'raw_pmc_nlm': nlm})
+            except Exception as e:
+                add_error('PubMed Central', 'fetch', exception=e)
+                traceback.print_exc()
+            else:
+                add_success('PubMed Central', 'fetch')
 
         ###########################################################################################
 
@@ -265,7 +350,106 @@ class Metadata(utopia.document.Annotator):
         common.utils.store_metadata(document, **scraped)
 
 
-    def reducePopulate(self, document):
+
+
+    def after_load_event(self, document):
+        # Put errors together in a sensible way
+        errors = {}
+        failures = 0
+        successes = 0
+        for error in document.annotations('errors.metadata'):
+            if error.get('concept') == 'Success':
+                successes += 1
+            elif error.get('concept') == 'Error':
+                failures += 1
+
+            component = error.get('property:component')
+            errors.setdefault(component, {})
+
+            category = error.get('property:category')
+            errors[component].setdefault(category, [])
+
+            method = error.get('property:method')
+            message = error.get('property:message', '')
+            errors[component][category].append((method, message))
+        categories = {}
+        for component, details in errors.iteritems():
+            for category in details.keys():
+                categories.setdefault(category, 0)
+                categories[category] += 1
+
+        # If there are errors, provide feedback to the user
+        if failures > 0:
+            # Check for likely client problems
+            if categories.get('connection', 0) == failures and successes == 0:
+                summary = '''
+                    Utopia could not reach any of the online services it would
+                    normally use to identify this document, meaning you are
+                    likely to see limited or no information below. You might
+                    wish to check your Internet connection and reload the
+                    document.
+                    '''
+            elif categories.get('timeout', 0) > 1:
+                if categories.get('timeout', 0) == failures and successes == 0:
+                    many = ''
+                else:
+                    many = 'some of'
+                summary = '''
+                    Utopia gave up contacting {0} the online services it would
+                    normally use to identify this document because they were
+                    taking too long to respond. You are likely to see limited
+                    or no information below. You might wish to check your
+                    Internet connection and reload the document.
+                    '''.format(many)
+            else:
+                if failures == 1:
+                    noun = 'An error'
+                else:
+                    noun = 'Errors'
+                summary = '''
+                    {0} occurred when trying to discover the identity
+                    of this document. You are likely to see limited or no
+                    information below.
+                    '''.format(noun)
+            html = '''
+                <div class="box error">
+                    <strong>Warning</strong>
+                    <p>
+                        {0}
+                    </p>
+                    <div class="expandable" title="Details...">
+                    <ul>
+            '''.format(summary)
+            for component, details in errors.iteritems():
+                for category, methods in details.iteritems():
+                    if category != 'success':
+                        summary = {
+                            'timeout': '{0} did not respond',
+                            'connection': 'Could not connect to {0}',
+                            'server': '{0} behaved unexpectedly',
+                        }.get(category, 'An error occurred accessing {0}')
+                        methods_html = ', '.join(('<span title="{1}">{0}</span>'.format(method, message) for method, message in methods))
+                        html += '<li>{0} (when accessing: {1}).</li>'.format(summary.format('<strong>' + component + '</strong>'), methods_html)
+            html += '''
+                    </ul>
+                    </div>
+                <div>
+            '''
+            annotation = spineapi.Annotation()
+            annotation['concept'] = 'Collated'
+            annotation['property:html'] = html
+            annotation['property:name'] = 'Error'
+            annotation['session:weight'] = '1000'
+            annotation['session:default'] = '1'
+            annotation['session:headless'] = '1'
+            document.addAnnotation(annotation)
+
+        print errors
+
+
+
+
+    def after_ready_event(self, document):
         # Make an annotation for all these metadata
         ids = {
             'doi': ('DOI', u'<a href="http://dx.doi.org/{0}">{0}</a>'),
